unit udmsqldb;

{$mode ObjFPC}{$H+}

//Switches:
//You can disable switches by using a period after the curved
//bracket like this  {.$DEFINE SWITCHNAME}

//delete Databasefile before ZConnection1.Open so every start an new and empty file is created
{.$DEFINE DeleteDatabasefileBeforeOpen}

//Import Data automatically on Create. You can use the procedure ImportData to import
//data at any given time
{.$DEFINE ImportDemoDataOnCreate}

//*********extended settings:

{By default, based on the selected connection type, the default library
for that connection type is used. If you want to use your own library,
activate this switch. The name of the own library has to be entered under
cSQLdbLibraryName in the include file. Note the differences in the name
under Windows and Linux.}
{.$DEFINE UseCustomLibrary}

{If you want to use a SQL log  enable the switch}
{.$DEFINE UseSQLLOG}

{Note: an extra possibility to run queries that have no result like  INSERT, DELET or DDL language
 - SQLQuery1 is provided.
Create your own SQLQuery-Components as well as own Datasource-Components in your forms and
connect them with the SQLConnector1
}



interface

uses
  Classes, SysUtils, SQLDBLib, SQLDB, Forms, uguessfile
  {$ifdef UseSQLLOG}
  , LazLogger
  {$endif}
  , SQLite3Conn
  //,pqconnection     //PostgreSQ
  //,ibconnection     //Interbase ,Firebird
  //,mssqlconn        //MSSQL Server , Sybase
  //,oracleconnection //Oracle
  //,odbcconn         //ODBC Access
  //,mysql40conn      //different MySQL Versions as well as MariaDB
  //,mysql41conn
  //,mysql50conn
  //,mysql51conn
  //,mysql55conn
  //,mysql56conn
  //,mysql57conn
  //,mysql80conn
  ;

const
  {$include demosqlite.Inc}
  cSQLiteType='SQLite3';

//TSQLServerType  copied from SQLDBToolsunit.pas therefore they may be unsynchronized
type
  TSQLServerType = (ssFirebird, ssInterbase, ssMSSQL, ssMySQL, ssOracle, ssPostgreSQL, ssSQLite, ssSybase, ssUnknown);

type

  { Tdmsqldb }

  Tdmsqldb = class(TDataModule)
    SQLConnector1: TSQLConnector;
    SQLDBLibraryLoader1: TSQLDBLibraryLoader;
    SQLQuery1: TSQLQuery;
    SQLScript1: TSQLScript;
    SQLTransaction1: TSQLTransaction;
    {$ifdef UseSQLLOG}
    procedure DoSQLLog(Sender: TSQLConnection; EventType: TDBEventType; const Msg: String);
    {$endif}
    procedure DataModuleCreate(Sender: TObject);
  private
    SQLServerType : TSQLServerType;
  public
    procedure Initialize;
    procedure ImportData(sImportFile: string='');
    procedure OpenConnection;
    procedure CloseConnection;

    {lists the available connection types depending on the SQL units included in the uses section}
    function ListConnectionTypes:string;

    {determines the standard library for a given connection type depending on the SQL units included in the uses section }
    function GetStandardLibraryName(sConnector:string):string;

    {determines and loads the SQL access library depending on the different operating systems and the selected connection type}
    procedure MyLoadLib(sConnectionType,sLibname : String);
    function MyGetLibraryName:string;

    {returns the current SQLite Version if SQLite is chosen as ConnectorType}
    function GetSQLiteVersion:string;

  end;

var
  dmsqldb: Tdmsqldb;

implementation

uses
  dialogs;
{$R *.lfm}

{ Tdmsqldb }

procedure Tdmsqldb.DataModuleCreate(Sender: TObject);
begin
  Initialize;

  {$ifdef ImportDemoDataOnCreate}
    //Import SQL File (demo data, database structure etc)
    ImportData;
  {$endif}
end;

procedure Tdmsqldb.Initialize;
var
  sLibrary:string;
  sSQLfile:string;
begin
  CloseConnection;                                    //in case SQLDBLibraryLoader1 or SQLConnector1 were active in design mode close them

  //First
  sLibrary := MyGetLibraryName;
  if sLibrary = cUNDEF then exit;                     //for security reasons in case the exception fails

  //Second: load library
  MyLoadLib(cConnectorType,sLibrary);

  //Third:
  // Try to find the Database file for file based systems or set credentials for others.
  // If database file for file bases systems do not exist create the named database file automatically

  //distinguish file databases versus server based databases
  case SQLServerType of
    ssSQLite,
    ssInterbase,
    ssFirebird  : begin
      {$ifdef DeleteDatabasefileBeforeOpen}
       sSQLFile := GuessSQLDatabaseFile(true);
      {$else}
       sSQLFile := GuessSQLDatabaseFile(false);
      {$endif}
      SQLConnector1.DatabaseName:=sSQLFile;
    end
    else begin
      //feel free to get credentials elsewhere - from an INI file for example
      SQLConnector1.DatabaseName:=cSQLDatabasename;
      SQLConnector1.UserName:=cSQLuser;
      SQLConnector1.Password:=cSQLpassword;
      SQLConnector1.Hostname:=cSQLhostname;
    end;
  end;

  SQLConnector1.ConnectorType:=cConnectorType;
  SQLConnector1.CharSet:='UTF8';
  SQLConnector1.LoginPrompt := False;
  SQLConnector1.KeepConnection := False;
  SQLConnector1.Options:=[scoApplyUpdatesChecksRowsAffected] ;

  //+[sqImplicitTransaction];
  {
    TConnOption = (sqSupportParams, sqSupportEmptyDatabaseName, sqEscapeSlash, sqEscapeRepeat, sqImplicitTransaction, sqLastInsertID, sqSupportReturning,sqSequences);
  TConnOptions= set of TConnOption;


  Both sqoKeepOpenOnCommit and CommitRetaining will keep the result set open but they are not the same for transaction handling.

  Using sqoKeepOpenOnCommit with Commit will keep the result set and close the transaction.

  Using CommitRetaining will keep the result set and open a new transaction. I do not recommend using CommitRetaining unless you have a good reason for keeping an open transaction.

 ******************
 uses Classes, Sysutils, SQLDB (* und weitere Units: z.B. SQLite3conn *);
 var
   ListOfConnectors: TStrings;
   ConnectorTypeName: String;
   ConnectorDefinition: TConnectionDef;
 begin
   ListOfConnectors := TStringList.Create;
   for ConnectorTypeName in ListOfConnectors do
   begin
     ConnectorDefinition := GetConnectionDef(ConnectorTypeName);
     WriteLn(ConnectorDefinition.TypeName);
     WriteLn(ConnectorDefinition.Description);
     WriteLn(ConnectorDefinition.DefaultLibraryName);
   end;
   ListOfConnectors.Destroy;
 end.
 ******************
  }

  {$ifdef UseSQLLOG}
  SQLConnector1.OnLog:=@DoSQLLog;
  SQLConnector1.LogEvents:=LogAllEventsExtra;  //either LogAllEvents or LogAllEventsExtra
  {$endif}

  try
    SQLConnector1.Open;
  except
    On E : Exception do
      begin
        showmessage( LineEnding
                    + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                    + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                    + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                    + 'Connection to Database failed '  + LineEnding+ LineEnding
                    + E.Message);
        Exit;
      end;
  end;
end;

procedure Tdmsqldb.ImportData(sImportFile: string);
begin
  if sImportfile = '' then begin
     sImportFile := GuessSQLImportFile;
     if sImportFile = cUNDEF then begin //just in case
        showmessage( 'SQL import file not found');
        exit;
     end
  end;
  if not FileExists(sImportFile) then begin
     showmessage('ImportData: File ' + sImportFile +'does not exist');
     exit;
  end;
  try
    SQLScript1.Script.LoadFromFile(sImportFile);
    SQLScript1.ExecuteScript;     //oder.Execute???
    SQLTransaction1.Commit;
  finally
    //
  end;

end;


procedure Tdmsqldb.OpenConnection;
begin
  if not SQLConnector1.Connected then SQLConnector1.Open;
end;

procedure Tdmsqldb.CloseConnection;
begin
    if SQLConnector1.Connected then SQLConnector1.Close;
end;

function Tdmsqldb.ListConnectionTypes: string;
Var
  S : TStringList;
  I : Integer;

begin
  S:=TStringList.Create;
  try
    getConnectionList(S);
    result:=LineEnding +'Available connection types:';
    For I:=0 to S.Count-1 do
      result:= result + LineEnding + S[i] + ', Default library name: ' + GetConnectionDef(S[i]).DefaultLibraryName;
  finally
    S.free;
  end;

end;

function Tdmsqldb.GetStandardLibraryName(sConnector: string): string;
var
  S : TStringList;
  I : Integer;
begin
  result:=cUNDEF;
  S:=TStringList.Create;
  try
    getConnectionList(S);
    For I:=0 to S.Count-1 do
      if S[i] = sConnector then begin
        result := GetConnectionDef(S[i]).DefaultLibraryName;
        exit;
      end;
  finally
    S.free;
  end;

end;

procedure Tdmsqldb.MyLoadLib(sConnectionType, sLibname: String);
begin
  //Using Loadlibrary and unloadlibrary instead of enables switch to catch exceptions
  try
    SQLDBLibraryLoader1.ConnectionType:=sConnectionType;
    SQLDBLibraryLoader1.UnLoadLibrary;
  except
    On E : Exception do
      begin
        showmessage( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'Error unloading library : '+ SQLDBLibraryLoader1.LibraryName
                          + LineEnding + LineEnding
                          + E.Message);

        Exit;
      end;
  end;

  SQLDBLibraryLoader1.ConnectionType:=sConnectionType; //define connector Type
  SQLDBLibraryLoader1.LibraryName:=sLibname;
  try
    SQLDBLibraryLoader1.LoadLibrary;
  except
    On E : Exception do
      begin
        showmessage( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'Error loading library : '+ sLibName
                          + LineEnding + LineEnding
                          + E.Message);

        Exit;
      end;
  end;

end;

function Tdmsqldb.MyGetLibraryName: string;
begin
  result:=cUNDEF;
   {$IFDEF UNIX}
     {$ifdef RasPi}
       result = '????';
     {$else}
       {$ifdef DARWIN}

         {$ifdef UseCustomLibrary}
           result='/usr/lib/'+cSQLdbLibraryName;
         {$else}
           result='/usr/lib/'+GetStandardLibraryName(cConnectorType);
         {$endif}

       {$else}

         {$ifdef UseCustomLibrary}
           result :=cSQLdbLibraryName;
         {$else}
           result := GetStandardLibraryName(cConnectorType);
         {$endif}

       {$endif}
     {$endif}
   {$ELSE}       //windows OS
     {$ifdef UseCustomLibrary}
       result := GuessLibraryLocation(cSQLdbLibraryName);
     {$else}
       result :='';
       result := GetStandardLibraryName(cConnectorType);
       result := GuessLibraryLocation;                   //Search for sqlite Library
     {$endif}
   {$ENDIF}
end;

function Tdmsqldb.GetSQLiteVersion: string;
var
  SQLQ:TSQLQuery;
begin
  result:='No SQLite3 Connection';

  if SQLConnector1.ConnectorType=cSQLiteType then begin
    SQLQ:=TSQLQuery.Create(nil);
    try
      SQLQ.DataBase := SQLConnector1;
      SQLQ.SQL.Text:='SELECT sqlite_version() AS version';
      SQLQ.Open;
      SQLQ.First;
      result:=SQLQ.FieldByName('version').AsString;
      SQLQ.Close;
    finally
      freeandnil(SQLQ);
    end;
  end;

end;


end.

