unit uguessfile;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, forms, dialogs{$ifdef WINDOWS},WinDirs{$endif};

const
  {$include demosqlite.Inc}

function GuessFile(const sFileName: string; const sWhereToSearch: string;
  var sSearched: string): string;      //tries to find a file in application dirextory and other directories
function GuessLibraryLocation(sLibraryName:string=''):string; //versucht die SQLite-DLL in Standardverzeichnissen zu suchen und gibt den Pfad+Dateinamen zurück
function GuessSQLImportFile(bAllowOpenDialog:boolean=false):string;    //versucht in standardverzeichnissen das ImportFile für FileDatenbanken (SQLITE etc) zu finden
function GuessSQLDatabaseFile(bDeleteOldFile:boolean=false):string;  //versucht in standardverzeichnissen das Datenfile für FileDatenbanken (SQLITE etc) zu finden
function GetTargetCPUinfo:string;

implementation

function GuessFile(const sFileName: string; const sWhereToSearch: string;
  var sSearched: string): string;
var
  s:string;
  sl:TStringlist;
  i: integer;
begin
  result:=cUNDEF;

  s :=  Application.Location + sFileName;
  if FileExists(s) then
  begin
   result := s;
   exit;
  end;
  sSearched:=sSearched + s + LineEnding;

  sl := TStringList.Create;
  try
    sl.Delimiter:=cDelimiter;
    sl.DelimitedText := sWhereToSearch;
    for i := 0 to sl.Count - 1 do begin
      s :=  Application.Location + sl[i]+ DirectorySeparator + sFileName;
      if FileExists(s) then
      begin
       result := s;
       break;
      end;
      sSearched:=sSearched + s + LineEnding;
    end;
  finally
    sl.Free;
  end;

end;

function GuessLibraryLocation(sLibraryName:string):string;
var
  s: string;
  sSearched:string;
  sPathString:string;
  sLibName:string;
begin
  result:=cUNDEF;
  sSearched:='';

  if sLibraryName='' then
    sLibName:=cSQLdbLibraryName
  else
    sLibName:=sLibraryName;

  {$ifdef Unix}
  sPathString:=''//no directories unser Linux necessary
  {$else}
  sPathString := 'dll@sql';
  {$endif}

  result:=GuessFile(sLibName,sPathString,sSearched);

  //Search in default directories
  {$if Defined(WINDOWS)}
  if result=cUNDEF  then begin
    if GetTargetCPUinfo = 'i386' then
        result :=  GetWindowsSpecialDir(CSIDL_WINDOWS,false) + 'sysWOW64'+ DirectorySeparator  + sLibName
    else
       result :=  GetWindowsSpecialDir(CSIDL_SYSTEM,false)  + sLibName;

  end;
  sSearched:=sSearched+ LineEnding+result;
  {$elseif Defined(Linux)}
    //regrettably i use Windows only
  {$elseif Defined(Darwin)}

  {$endif}

  if not FileExists(result) then  result := cUNDEF;

  if result=cUNDEF  then
  raise Exception.Create( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'tried to find Library ' + cSQLdbLibraryName + ' in '
                          + LineEnding + LineEnding
                          + sSearched
                          + 'but it was not found!' + LineEnding+ LineEnding
                          + 'Please provide ' + cSQLdbLibraryName +' in one of the searchpaths');

end;

{
The default import file or a file defined by the developer is searched and used.
If the user is allowed to select an import file set bAllowOpenDialog to true;
}
function GuessSQLImportFile(bAllowOpenDialog: boolean): string;
var
  s:string;
  sSearchedin:string;
  sPathString:string;
  OpenDialog:TOpendialog;
begin
  result:=cUNDEF;
  sSearchedin:='';

  {$ifdef Unix}
  sPathString:='to define'
  {$else}
  sPathString := 'dll@data';
  {$endif}

  result:=GuessFile(cSQLCustomImportFileName,sPathString,sSearchedin);
  if result=cUNDEF  then
    result:=GuessFile(cSQLStandardImportFileName,sPathString,sSearchedin);


  if (result=cUNDEF) and (bAllowOpenDialog) then begin
    OpenDialog := TOpendialog.Create(nil);
    OpenDialog.DefaultExt:='*.sql';
    try
      if OpenDialog.Execute then
        result := OpenDialog.Filename
      else
        result:=cUNDEF;
    finally
      freeandnil(OpenDialog);
    end;
  end;


  if result=cUNDEF  then
  raise Exception.Create( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'tried to find SQL import File '
                          + 'either' + cSQLCustomImportFileName + ' or ' + cSQLStandardImportFileName + ' in '
                          + LineEnding + LineEnding
                          + sSearchedin
                          + 'but it was not found!' + LineEnding+ LineEnding
                          + 'Please provide ' + 'either' + cSQLCustomImportFileName + ' or ' + cSQLStandardImportFileName +' in one of the searchpaths');

end;

function GuessSQLDatabaseFile(bDeleteOldFile: boolean): string;
var
  s:string;
  sSearchedin:string;
  sPathString:string;
begin
  result:=cUNDEF;
  sSearchedin:='';

  {$ifdef Unix}
  sPathString:='to define'
  {$else}
  sPathString := 'data@sql';
  {$endif}

  result:=GuessFile(cSQLDatabaseFile,sPathString,sSearchedin);

  if result=cUNDEF  then begin
    showmessage( LineEnding
                + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                + 'tried to find SQL data File in'
                + LineEnding + LineEnding
                + sSearchedin
                + 'but it was not found!' + LineEnding+ LineEnding
                + 'Please provide '  + cSQLDatabaseFile +' in one of the searchpaths'+ LineEnding+ LineEnding
                +' creating standard database file '+cSQLDatabaseFile+' in application directory instead');

    result:=cSQLDatabaseFile;
  end;

  if bDeleteOldFile then begin
    if FileExists(result) then DeleteFile(result);
    //if you want extra information uncomment next line.
    //showmessage('old database file was deleted and will be created as fresh new file');
  end;

end;


function GetTargetCPUinfo: string;
begin
  result := {$I %FPCTARGETCPU%};

{
if GetTargetInfo = 'i386' then
    ZConnection2.LibraryLocation:=(ExtractFilePath(Application.ExeName)+'32\sqlite3.dll')
else
   ZConnection2.LibraryLocation:=(ExtractFilePath(Application.ExeName)+'64\sqlite3.dll');
end;
}
end;

end.

